#!/usr/bin/perl -w
# $Id: ipUpdate.pl,v 1.2 2008/04/17 22:15:11 hiro Exp $
###################################################
##
##  DnsExit.Com Dynamic IP update script
##  v1.5
##
##################################################

use Http_get;

$PROGRAMDIR=`dirname $0`;
chop $PROGRAMDIR;
chdir($PROGRAMDIR) || die "Unable to go to dir $PROGRAMDIR";

#
# Get config variables
#
$cfile = "/etc/dnsexit.conf";
open (CFG, "< $cfile") || (print STDERR "Fail open config file $cfile. You need to run dns-setup.pl script" && exit );
while (<CFG>)
{
  (my $line = $_ ) =~ s/\s+$//;

  if ( length( $line ) < 2 || ( $line =~ /^\s*#/ ) )
  {
    next;
  }
  ($key, $value) = split(/=/, $line);
  $keyVal{$key} = $value;
}
$ipfile   = $keyVal{"cachefile"} || '/tmp/dnsexit-ip.txt';
$pidfile  = $keyVal{"pidfile"} || '/var/run/ipUpdate.pid';
$daemon   = lc($keyVal{"daemon"}) || 'yes';
$interval = $keyVal{"interval"} || 600;
$logfile  = $keyVal{"logfile"} || '/var/log/dnsexit.log';
$interface	= $keyVal{"interface"};
$altinterface	= $keyVal{"altinterface"};

if ($#ARGV >= 0 && $ARGV[0] eq '-D') {
	$daemon = 'no';
}

if ( ! ( $daemon eq "yes" ) )
{
  clear();
  if (defined($interface)) {
    $ip = getIFIP($interface);
	if (!validIP($ip) && defined($altinterface)) {
      $ip = getIFIP($altinterface);
	}
	if (!validIP($ip)) {
      $ip = getProxyIP();
	}
  } else {
    $ip = getProxyIP();
  }
  
  $ipFlag = isIpChanged($ip);
  if ( $ipFlag == 1 )
  {
    mark("INFO", "100", "IP is not changed from last successful update");
    exit 0;
  }
  postNewIP( $ip );
}
else
{
  daemonize();
  while(1)
  {
    clear();
    mark("INFO", "100", "Started in daemon mode");
    $ip = defined($interface)	? getIFIP($interface)	: getProxyIP();
    $ipFlag = isIpChanged($ip);
    if ( $ipFlag == 1 )
    {
      mark("INFO", "100", "IP is not changed from last successful update");
    } 
    else
    {
      postNewIP( $ip );
    }
    sleep( $interval );
  }
}
exit 0;
#-----------------------------------------------------
#-- Sub Routines
#-----------------------------------------------------
sub postNewIP
{
  my $newip = shift ( @_ );
  my $get = new Http_get;
  my $url = $keyVal{"url"};
  my $login = $keyVal{"login"};
  my $password = $keyVal{"password"};
  my $host = $keyVal{"host"};
  my $posturl = "${url}?login=${login}&password=${password}&host=${host}";

  if ( $newip =~ /\d+\.\d+\.\d+\.\d+/ )
  {
    $posturl = ${posturl} . "&myip=${newip}";
  }

  my $response = $get->request($posturl);
  if ($response->is_success)
  {
    #record successful update of the ip address
    
    $result = $response->content;    
    if ( $result =~ /(\d+)=(.+)/ )
    {
      mark("Success", "$1", "$2");
      open S, "> $ipfile";
      print S $ip;
      close S;
    }
    else
    {
      mark("ERROR", "-99", "Return content format error");
    }

  }
  else
  {
    mark("ERROR", $response->code, $response->message);
  }

}

sub isIpChanged
{
  my $newip = shift(@_);
  open SS, "< $ipfile";
  my $preip = <SS>;
  close SS;
  #print "new=[$newip] old=[$preip]";
  if (!($newip eq $preip))
  {
    return 0;
  }
  return 1;
}
sub getProxyIP
{
  my $get = new Http_get;
  my $ipServs = $keyVal{"proxyservs"};
  my @servs = split(/;/, $ipServs);
  foreach $server ( @servs )
  {
    $myUrl = "http://" . $server;
    
    $response = $get->request($myUrl);
    if ($response->is_success)
    {
    	if ( $response->content =~ /\D*(\d+\.\d+\.\d+\.\d+).*/ )
    	{
    		mark("INFO", "100", "$myUrl says your IP address is: $1");
    		return ( $1 );
    	}
    	else
    	{
    		mark("ERROR", "-100", "Return message format error.... Fail to grep the IP address from ".$myUrl);
    	}
    }
  }
  mark("ERROR", "-99", "Fail to get the proxy IP of your machine");
  return "";
}
sub mark
{
    my ($type, $code, $message) = @_;
    open (LOGFILE, ">>$logfile");
    my $msg=localtime()."\t$type\t".$code."\t".$message."\n";
    print $msg;
    print LOGFILE $msg;
    close LOGFILE;
}
sub clear
{
    open (LOGFILE, "> $logfile");
    close LOGFILE;
}

# Daemonize the process and write pid to pidfile
sub daemonize {
  open (STDIN, '/dev/null')   or die "Can't read /dev/null: $!";
  open (STDOUT, '>>/dev/null') or die "Can't write to /dev/null: $!";
  open (STDERR, '>>/dev/null') or die "Can't write to /dev/null: $!";
  defined(my $pid = fork)   or die "Can't fork: $!";
  if ($pid ) {
    open (PIDFILE, ">$pidfile");
    print PIDFILE $pid;
    close PIDFILE;
    exit;
  }
  umask 0;
}

#
# Get specified interface's IP
#
sub getIFIP {
	my ($if)	= @_;
	my $addr	= '0.0.0.0';
	unless (open IP, "ip addr show $if |") {
		mark("ERROR", "-101", "Can't invoke ip command");
		exit 1;
	}

	while (<IP>) {
		chomp;
		if (/^\s*inet (\d+\.\d+\.\d+\.\d+)\/\d{1,2} brd \d+\.\d+\.\d+\.\d+ scope global $if\s*$/) {
			$addr	= $1;
			last;
		}
	}
	close IP;
	return $addr;
}

#
# Check if a valid IP (non private, etc.)
#
sub validIP {
	my @i	= split /\./, (shift @_);
	($#i != 3)	&& return 0;
	($i[0] == 0)	&& return 0;		# Current
	($i[0] == 10)	&& return 0;		# Private
	($i[0] == 14)	&& return 0;		# Public
	($i[0] == 127)	&& return 0;		# Loopback
	($i[0] == 128 && $i[1] == 0)	&& return 0;				# Reserved
	($i[0] == 169 && $i[1] == 254)	&& return 0;				# Link-local
	($i[0] == 172 && (16 <= $i[1] && $i[1] <= 31))	&& return 0;#Private
	($i[0] == 191 && $i[1] == 255)	&& return 0;				# Reserved
	($i[0] == 192 && $i[1] == 0 && $i[2] == 0)	&& return 0;	# Reserved
	($i[0] == 192 && $i[1] == 0 && $i[2] == 2)	&& return 0;	# Reserved
	($i[0] == 192 && $i[1] == 88 && $i[2] == 99)	&& return 0;# IPv6
	($i[0] == 192 && $i[1] == 168)	&& return 0;				# Private
	($i[0] == 192 && ($i[1] == 18 || $i[1] == 19))	&& return 0;# Benchmark
	($i[0] == 223 && $i[1] == 255 && $i[2] == 255)	&& return 0;# Reserved
	(224 <= $i[0] && $i[0] <= 239)	&& return 0;				# Multicast
	(240 <= $i[0] && $i[0] <= 255)	&& return 0;				# Reserved
	($i[0] == 255 && $i[1] == 255 && $i[2] == 255 && $i[3] == 255)	&& return 0;
	return 1;
}
